	real*8 function peepiX(vertex,main)


* Sept. 2003 D. Gaskell
* Purpose:
* This routine calculates p(e,e'pi+)X semi-iinclusive cross sections from
* the CTEQ5M parton distribution functions and a simple parameterization
* of the favored and unfavored fragmentation functions.
*   output:
*	sigma_eepiX	!d3sigma/dEe'dOmegae'Omegapi	(microbarn/MeV/sr^2)
* 
* For now, just does PI+ !!!!!!!!!!!!!!!!!!
* 
* October 3 2003 D. Gaskell
* Replace simple fragmentation function paramterization with a slightly
* more sophisticated treatment from Binnewies et. al.,PRD 52, p.4947 (1995).
* This parameterization gives: D_u^(pi+ + pi-) = D_d^(pi+ + pi-) = D+ + D-
* The separate favored and unfavored fragmentation functions are given by
* the ratio D-/D+ from HERMES data (my fit to P. Geiger's results).
* The D-/D+ fit is valid from z=0.25 to 0.9 or so, but behaves pretty well
* at lower z (approaches simple form of Field and Feynman (1-z)/(1+z)).
*
*
* October 4 2003 D. Gaskell
* Add PI- functionality
*
* October 9 2003 D. Gaskell
* Add deuterium functionality.  Assume just an incoherent sum of parton
* distributions from proton and neutron.  Assume isospin symmetry so
* d_neutron(x) = u_proton(x)
* dbar_neutron(x) = ubar_proton(x)
* u_neutron(x) = d_proton(x)
* ubar_neutron(x) = dbar_proton(x)



	implicit none
	include 'simulate.inc'

	record /event_main/ main
	record /event/ vertex

* PDFs
	integer iset  !which set (1=cteq5m)
	integer ipart !particle u=1, ubar=-1, d=2, dbar=-2
	real*8 u,d,ubar,dbar
	real*8 qu,qd ! u and d quark charges
	real*8 D_fav, D_unfav, D_sum, R_D !favored,unfavored,sum,ratio of FFs
	real*8 lambda, Q2zero ! scales for FF param
	real*8 sv !scaling variable for FF param
	real*8 N,a1,a2 !parameters for FF param

	real*8 xbj,y,sx, Q2gev, Qgev, pt2gev
	real*8 b  ! pt2 parameter for FFs

	real*8 sum_sq, dsigdz, sigsemi, jacobian, sigma_eepiX
	real*8 sighad, sige

	real*8 F1,F2,W1,W2,sin2th2,cos2th2,W2coeff
	real*8 Ctq5Pdf	
c	external Ctq5Pdf

	logical first


	parameter (iset=1)
	parameter (qu=2./3.)
	parameter (qd=-1./3.)

	parameter (b=3.76)   !GeV^-2

	parameter (lambda=0.227) !0.227 GeV for NLO
	parameter (Q2zero=2.0)   !Gev^2 for u,d,s,g

	data first /.TRUE./

	if(first) then
	   call SetCtq5(iset)    ! initialize Cteq5 (we're using cteq5m)
	   first=.FALSE.
	endif

	sx = (2.*vertex.Ein*mp + mp**2)/1.d6  !convert to GeV2
	xbj= vertex.Q2/2./mp/vertex.omega   
	if(xbj.gt.1.0) then
	   write(6,*) 'XBj is too large!', xbj
	   xbj=1.0
	endif
	y = vertex.omega/vertex.Ein

C DJG convert some stuff to GeV

	Q2gev = vertex.q2/1.d6
	Qgev = sqrt(Q2gev)
	pt2gev = vertex.pt2/1.d6

C Get the PDFs
	ipart=1
	u = Ctq5pdf (ipart , xbj, Qgev)

	ipart=-1
	ubar = Ctq5pdf (ipart , xbj, Qgev)

	ipart=2
	d = Ctq5pdf (ipart , xbj, Qgev)

	ipart=-2
	dbar = Ctq5pdf (ipart , xbj, Qgev)

C Simple paramaterization from Kretzer et al (EPJC 22 p. 269)
C for Q2=2.5.

c	D_fav = 0.689*vertex.zhad**(-1.039)*(1.0-vertex.zhad)**1.241
c	D_unfav = 0.217*vertex.zhad**(-1.805)*(1.0-vertex.zhad)**2.037

C

C Paramaterization from Binneweis et al
	if(Q2gev.gt.lambda**2) then

	   sv = log( log(Q2gev/lambda**2)/log(Q2zero/lambda**2) )

C Form of parameterization is D = N z^a1 (1-z)^a2

	   if(doing_semipi) then

	      N = 1.150 - 1.522*sv + 1.378*sv**2 - 0.527*sv**3
	      a1 = -0.740 - 1.680*sv + 1.546*sv**2 - 0.596*sv**3
	      a2 = 1.430 + 0.543*sv - 0.023*sv**2

	   elseif(doing_semika) then

	      STOP 'Kaon fragmentation functions not implemented yet'
	      
	   endif

	   D_sum = N*vertex.zhad**a1*(1.0-vertex.zhad)**a2

	else
	   write(6,*) 'Q2 is less than', lambda**2, 'GeV2'
	   write(6,*) 'Setting Frag. Function to zero!'
	   D_sum = 0.0
	endif

C Ratio of D-/D+ from P. Geiger's thesis (HERMES)

	R_D = (1.0-vertex.zhad)**0.083583/(1.0+vertex.zhad)**1.9838

	D_fav = D_sum/(1.0+R_D)
	D_unfav = D_sum/(1.0+1.0/R_D)
	
	sum_sq = qu**2*(u+ubar) + qd**2*(d+dbar)

	if (doing_deutsemi) then
	   sum_sq = sum_sq + qu**2*(d+dbar) + qd**2*(u+ubar)
	endif

c	write(6,*) 'semi stuff', D_sum,N,a1,a2,sv,vertex.zhad,Q2gev
	if(sum_sq.gt.0.) then
	   if(doing_hplus) then
	      dsigdz = (qu**2*u*D_fav + qu**2*ubar*D_unfav +
	1	   qd**2*d*D_unfav + qd**2*dbar*D_fav)/sum_sq
	   else
	      dsigdz = (qu**2*u*D_unfav + qu**2*ubar*D_fav +
	1	   qd**2*d*D_fav + qd**2*dbar*D_unfav)/sum_sq
	   endif
	   if(doing_deutsemi) then
	      if(doing_hplus) then
		 dsigdz = dsigdz + (qu**2*d*D_fav + qu**2*dbar*D_unfav +
	1	      qd**2*u*D_unfav + qd**2*ubar*D_fav)/sum_sq
	      else
		 dsigdz = dsigdz + (qu**2*d*D_unfav + qu**2*dbar*D_fav +
	1	      qd**2*u*D_fav + qd**2*ubar*D_unfav)/sum_sq
	      endif
	   endif
	else
	   dsigdz = 0.0
	endif

	sighad = dsigdz*b*exp(-b*pt2gev)/2./pi

c	sige = alpha**2*(1.+(1.-y)**2)*(vertex.e.E/1000.)/
c	1    (sx*xbj*y**2*(mp/1000.)*(vertex.omega/1000.))
c	2    * sum_sq


	F2 = xbj*sum_sq
	F1 = F2/2./xbj

	W1 = F1/(mp/1000.)
	W2 = F2/(vertex.omega/1000.)

	sin2th2 = vertex.Q2/4./vertex.Ein/vertex.e.p
	cos2th2 = 1.-sin2th2
	W2coeff = cos2th2

	sige = 4.*alpha**2*(vertex.e.p/1000)**2/Q2gev**2 * ( W2*W2coeff + 2.*W1*sin2th2)

c	write(6,*) 'e-Kin:Q2,th_E,Pe',Q2gev,vertex.e.theta*180./3.14159,vertex.e.P
c	write(6,*) 'electron cross section (ub/GeV/sr)',
c	1    sige*(hbarc/1000.)**2*10000
C DJG This dsig/(dOmega_e dE_e dz dpt**2 dPhi_had) in microbarn/GeV**3/sr 
	sigsemi = sige*sighad*(hbarc/1000.)**2*10000.0


c	write(6,*) 'Had-kin:z,pt2',vertex.zhad,pt2gev
c	write(6,*) 'Frag:Dplus,Dminus',D_fav,D_unfav
C Need to convert to dsig/ (dOmega_e dE_e dE_h dCos(theta) dPhi_had
C This is just given by 1/omega * 2*p_h**2*cos(theta)

	jacobian = 1./(vertex.omega/1000.)*2.*(vertex.p.P/1000.)**2
	1    *cos(vertex.theta_pq)

C The 1.d6 converts from microbarn/GeV^2 to microbarn/MeV^2
	sigma_eepiX = sigsemi*jacobian/1.d6

	

	main.davejac = jacobian*1000.0  ! Convert Jacobian from GeV -> MeV
	ntup.sigcm = sighad

	peepiX = sigma_eepiX


	return

	end
